const io = require('socket.io');
const AuthService = require('./AuthService');
const UpService = require('./UpService');
const adqGetDashboardFiliais = require('../model/adqGetDashboardFiliais');
const parametrizacoesModel = require('../model/parametrizacoes');
const paginacao = require('../functions/Paginacao');
const monitoringController = require('../controller/MonitoringController');

class SocketService {
  monitoringClients = []; // Registrar os portais
  updaterClients = [];

  async start(http) {
    this.browserlist = {};
    this.slavelist = await this.getDashboardFiliais();
    this.defaultInitTimeout = 5000;
    this.intervals = []; // Registrar is intervals
    this.assIntervals = [];
    this.lastSocketId = null;
    this.masterSocket = null;
    this.portalSocketId = null;

    this.parametersError = [];
    this.vendasError = [];
    this.satError = [];
    this.monitoringResult = {};
    this.io = io(http, { cors: '*' });

    this.io.on('connection', (socket) => {
      this.masterSocket = socket;
      console.log('newconnection: ', socket.id);
      socket.emit('updateVersion', { serviceName: 'xxx' });

      /**
       * Atualiza informações do Dashboard, inclusive o ID do socket e registra a sala
       */
      socket.on('join', (room) => {
        this.slavelist.filter((filial) => {
          if(filial.id === room.id){
            Object.assign(filial, room);
          }
          return filial;
        });
        socket.join(room.room);
      });

      /** Garante que o portal está conectado */
      socket.on('ping', (data) => {
        const clintId = (data.id)? data.id : data;
        if(data.filial) {
          /** se for loja, atualiza os dados e avise o portal */
          this.slavelist.filter((filial) => {
            if(filial.id === data.filial) {
              /** Merge de info */
              Object.assign(filial, {socketId: data.id, situacao: 'Ativo'});
              /** Atualiza o Portal */
              this.io.to(this.portalSocketId).emit('login', {
                text: 'UNIDADE CONECTADA',
                error: false,
                message: {
                  data: this.slavelist,
                },
              });
            }
            return filial;
          });
        }

        /** Confere as conexões */
        this.monitoringClients.filter((portais) => {
          if(portais.id === clintId){
            Object.assign(portais, {time: new Date().getTime()});
          }
          return portais;
        });
      });

      /** Remove conexões anativas */
      this.intervals.push({
        id: socket.id,
        interval: setInterval(() => {
          this.monitoringClients.forEach((portais) => {
            if((portais.time + 1200000) < new Date().getTime()){
              /** Client não atualizou a conexão, mateo */
              this.io.sockets.sockets.forEach((socket) => {
                if(socket.id === portais.id) {
                  console.log('Desconectei o: ', socket.id);
                  socket.disconnect(true);
                }
              });
            }
          });
        }, 20000)
      });
      /** Garante que o portal está conectado */

      socket.on('login', (data) => {
        console.log('Login: ');
        this.login(socket, data);
      });

      /**
       * Exportação de vendas
       */
      socket.on('iniciaExportacao', (data) => {
        try {
          data.payload = JSON.parse(data.payload);
          UpService.setInsereServidor(data.CDFILIAL, data.CDCAIXA, data.CDLOJA, data.payload);
        } catch (e) {}
      });

      /**
       * Importação
       */
      socket.on('importar', (data) => {
        console.log('importar', data);
        let target = this.slavelist[socket.email][data.CDFILIAL];
        this.io
          .to(target.id)
          .emit('command', { type: 'import', data: { dataUrl: '/API/getStructure' } });
      });

      /**
       * Desconet do socket
       */
      socket.on('disconnect', () => {
        /** Limpa os Intervals */
        this.intervals.forEach((interval) => {
          if(interval.id === socket.id) {
            clearInterval(interval.interval);
          }
        });
        const indIntervals = this.intervals.findIndex((item) => {
          return item.id === socket.id;
        });
        if(indIntervals != -1) {
          this.intervals.splice(indIntervals);
        }

        /** Limpa as Conexões (Portais e Lojas) */
        const idxMonitoring = this.monitoringClients.findIndex((item) => {
          return item.id === socket.id;
        });
        if(idxMonitoring != -1) {
          this.monitoringClients.splice(idxMonitoring);

          const subsidiaryIndex = this.slavelist.findIndex((item) => {
            return item.socketId === socket.id;
          });
  
          if (this.slavelist[subsidiaryIndex]) {
            this.slavelist[subsidiaryIndex].situacao = 'Inativa';
            this.io.to(this.portalSocketId).emit('login', {
              text: 'UNIDADE DESCONECTADA',
              error: false,
              message: {
                data: this.slavelist,
              },
            });
          }
        } else {
          const idxUpdater = this.updaterClients.findIndex((item) => {
            return item.id === socket.id;
          });
          if(idxUpdater != -1) {
            this.updaterClients.splice(idxUpdater);
            this.io.to(this.portalSocketId).emit('updaterConnected', {
              CDFILIAL: socket.CDFILIAL,
              CONNECTED: false,
            });
          }
        }
      });

      socket.on('detailMonitoring', (config) => {
        const detail = this.monitoringResult[config.CDFILIAL] ? this.monitoringResult[config.CDFILIAL][config.ID] : [];
        this.io.to(socket.id).emit('detailMonitoringResult', detail);
      });

      socket.removeAllListeners('execMonitoring');
      socket.on('execMonitoring', (config) => {     
        // Socket das lojas conectadas
        this.monitoringClients.forEach((item) => {
          const filialIdx = this.slavelist.findIndex((filial) => {
            return filial.codigo === item.socket.CDFILIAL;
          });
          // A => Execução na base do Assistant
          // L => Execução na base Lojas, envia a requisição e aguarda o retorno para emitir para o Assistant
          // I => Execução de rotinas criadas internamente que não podem ser alteradas via sistema (SAT, por exemplo)
          // E => Execução de requisição HTTP nas Lojas, envia a requisição e aguarda o retorno para emitir para o Assistant
          // S => Verificação se Serviços do Windows na Loja estão em execução
          if(config.TIPO == 'A') {
            monitoringDataBaseCloud.bind(this)(config, filialIdx, item.socket);
          } else if(config.TIPO == 'L') {
            monitoringDataBaseLoja.bind(this)(config, filialIdx, item.socket);
          } else if(config.TIPO == 'I') {
            monitoringAssistantInterno.bind(this)(config, filialIdx, item.socket);
          } else if(config.TIPO == 'E') {
            monitoringApacheLoja.bind(this)(config, filialIdx, item.socket);
          } else if(config.TIPO == 'S') {
            monitoringServicesLoja.bind(this)(config, filialIdx, item.socket);
          }
        });
      });

      function monitoringDataBaseCloud(config, filialIdx, socketClient) {
        const params = { CDFILIAL: socketClient.CDFILIAL };
        monitoringController.execCommand(config.COMANDO, params).then((response) => {
          console.log('Filial:', socketClient.CDFILIAL, '| DB Cloud SUC:', config.ID + ' - ' + config.NOME, '| Quantidade:', response.rows.length);
          setMonitoringResult.bind(this)(socketClient, config, filialIdx, response, false);
        }).catch((error) => {
          const response = {
            metaData: [ { name: 'ERRO' } ],
            rows: [ [ error.message ] ]
          };
          console.log('Filial:', socketClient.CDFILIAL, '| DB Cloud ERR:', config.ID + ' - ' + config.NOME, '| Message:', error.message);
          setMonitoringResult.bind(this)(socketClient, config, filialIdx, response, true);
        });
      }

      function monitoringDataBaseLoja(config, filialIdx, socketClient) {
        //Envia para a Loja
        socketClient.emit('execMonitoringSQL', config, socketClient.rooms);

        //Recebe a resposta da Loja, sempre atualizando o listener para não duplicar
        socketClient.removeAllListeners(`${config.ID}_result`);
        socketClient.on(`${config.ID}_result`, (result) => {
          if(!result.isError) {
            console.log('Filial:', socketClient.CDFILIAL, '| SQL Loja SUC:', config.ID + ' - ' + config.NOME, '| Quantidade:', result.response.rows.length);
            setMonitoringResult.bind(this)(socketClient, config, filialIdx, result.response, false);
          } else {
            const response = {
              metaData: [ { name: 'ERRO' } ],
              rows: [ [ result.response ] ]
            };
            console.log('Filial:', socketClient.CDFILIAL, '| SQL Loja ERR:', config.ID + ' - ' + config.NOME, '| Message:', result.response);
            setMonitoringResult.bind(this)(socketClient, config, filialIdx, response, true);
          }
        });
      }

      function monitoringApacheLoja(config, filialIdx, socketClient) {
        //Envia para a Loja
        socketClient.emit('execMonitoringHttp', config, socketClient.rooms);

        //Recebe a resposta da Loja, sempre atualizando o listener para não duplicar
        socketClient.removeAllListeners(`${config.ID}_result`);
        socketClient.on(`${config.ID}_result`, (result) => {
          if(!result.isError) {
            const isError = result.response.rows.some((element) => {
              return element.STATUS == 'N';
            });
            if(!isError) {
              result.response.rows = [];
            }
            console.log('Filial:', socketClient.CDFILIAL, '| HTTP Loja SUC:', config.ID + ' - ' + config.NOME);
            setMonitoringResult.bind(this)(socketClient, config, filialIdx, result.response, isError);
          } else {
            const response = {
              metaData: [ { name: 'ERRO' } ],
              rows: [ [ result.response ] ]
            };
            console.log('Filial:', socketClient.CDFILIAL, '| HTTP Loja ERR:', config.ID + ' - ' + config.NOME, '| Message:', result.response);
            setMonitoringResult.bind(this)(socketClient, config, filialIdx, response, true);
          }
        });
      }

      function monitoringServicesLoja(config, filialIdx, socketClient) {
        //Envia para a Loja
        socketClient.emit('execMonitoringServices', config, socketClient.rooms);

        //Recebe a resposta da Loja, sempre atualizando o listener para não duplicar
        socketClient.removeAllListeners(`${config.ID}_result`);
        socketClient.on(`${config.ID}_result`, (result) => {
          if(!result.isError) {
            const isError = result.response.rows.some((element) => {
              return element.STATUS == 'N';
            });
            if(!isError) {
              result.response.rows = [];
            }
            console.log('Filial:', socketClient.CDFILIAL, '| Services Loja SUC:', config.ID + ' - ' + config.NOME);
            setMonitoringResult.bind(this)(socketClient, config, filialIdx, result.response, isError);
          } else {
            const response = {
              metaData: [ { name: 'ERRO' } ],
              rows: [ [ result.response ] ]
            };
            console.log('Filial:', socketClient.CDFILIAL, '| Services Loja ERR:', config.ID + ' - ' + config.NOME, '| Message:', result.response);
            setMonitoringResult.bind(this)(socketClient, config, filialIdx, response, true);
          }
        });
      }
      
      function monitoringAssistantInterno(config, filialIdx, socketClient) {
        if(config.NOME == 'SAT' && this.slavelist[filialIdx].sat !== 'N') {
          //Envia para a Loja
          socketClient.emit("satMonitoring", config, socketClient.rooms);

          //Recebe a resposta da Loja, sempre atualizando o listener para não duplicar
          socketClient.removeAllListeners('satMonitoringResult');
          socketClient.on('satMonitoringResult', (result) => {
            if(!result.isError) {
              const isSatError = result.response.rows.some((element) => {
                return !element.sat;
              });
              if(!isSatError) {
                result.response.rows = [];
              }
              console.log('Filial:', socketClient.CDFILIAL, '| Exec SAT SUC:', config.ID + ' - ' + config.NOME, '| Error:', isSatError);
              setMonitoringResult.bind(this)(socketClient, config, filialIdx, result.response, isSatError);
              if(this.slavelist[filialIdx]) {
                this.slavelist[filialIdx].sat = isSatError ? 'E' : 'S';
              }
            } else {
              const response = {
                metaData: [ { name: 'ERRO' } ],
                rows: [ [ result.response ] ]
              };
              console.log('Filial:', socketClient.CDFILIAL, '| Exec SAT ERR:', config.ID + ' - ' + config.NOME, '| Message:', result.response);
              setMonitoringResult.bind(this)(socketClient, config, filialIdx, response, true);
              if(this.slavelist[filialIdx]) {
                this.slavelist[filialIdx].sat = 'E';
              }
            }
          });
        }
      }

      function setMonitoringResult(socketClient, config, filialIdx, response, isError) {
        //Atualiza a lista a ser usada no grid de monitoramento.
        if(this.slavelist[filialIdx]) {
          this.slavelist[filialIdx][config.ID] = isError || response.rows.length > 0 ? 'N' : 'S';
        }
        //Atualiza os dados do modal de detalhes de cada monitoramento.
        this.monitoringResult[socketClient.CDFILIAL][config.ID] = response;
        
        //Envia informações atualizadas para a tela de monitoramento.
        this.io.to(socket.id).emit(config.ID, this.slavelist);
      }

      socket.removeAllListeners('execUpdater');
      socket.on('execUpdater', (params) => {
        const updaterSocketFind = this.updaterClients.find((item) => item.socket.CDFILIAL == params.CDFILIAL);
        if(updaterSocketFind) {
          const updaterSocket = updaterSocketFind.socket;
          // Envia requisição de atualização para a loja
          updaterSocket.emit('updater', params);
  
          // Recebe a resposta da execução
          updaterSocket.removeAllListeners('updaterResultInProgress');
          updaterSocket.on('updaterResultInProgress', (result) => {
            this.io.to(socket.id).emit('updaterInProgress', result);
          });
          updaterSocket.removeAllListeners('updaterResultFinish');
          updaterSocket.on('updaterResultFinish', (result) => {
            this.io.to(socket.id).emit('updaterFinish', result);
          });
        } else {
          console.log('CLIENT DO UPDATER NÃO ENCONTRADO NA FILIAL: ' + params.CDFILIAL);
        }
      });

      this.lastSocketId = socket.id;
    });
  }

  login(socket, data) {
    // Não é uma filial que está conectando e sim um portal
    if (!data.CDFILIAL) {
      AuthService.autenticar(data.email, data.password).then((result) => {
        socket.email = data.email;
        this.portalSocketId = socket.id;
        this.io.to(socket.id).emit('confirmLogin', { 
          error: false, 
          message: 'Você está conectado', 
          result: result 
        });

        if (!data.slave && !data.updater) {
          if (this.slavelist === undefined) {
            this.getDashboardFiliais(data).then((dashboard) => {
              this.io.to(socket.id).emit('login', {
                text: 'PORTAL CONECTADO',
                error: false,
                message: dashboard,
                result,
              });
            });
          } else {
            this.io.to(socket.id).emit('login', {
              text: 'PORTAL CONECTADO',
              error: false,
              message: this.slavelist,
              result,
            });
          }
        }
      }).catch((error) => {
        console.log(error);
        this.io.to(socket.id).emit('message', 'Você não é um usuário autorizado!');
        socket.disconnect();
      });
    } else {
      // Uma filial se coneta ao master (Updater ou Monitoring)
      AuthService.autenticar(data.email, '', data.CDFILIAL).then((result) => {
        socket.email = data.email;
        socket.CDFILIAL = data.CDFILIAL;

        // IF -> Conexão de client do Updater
        // ELSE -> Conexão de client do Monitoring
        if(data.updater) {
          this.updaterClients.push({
            id: socket.id,
            socket: socket,
            time: new Date().getTime(),
          });
          this.io.to(this.portalSocketId).emit('updaterConnected', {
            CDFILIAL: socket.CDFILIAL,
            CONNECTED: true,
          });
        } else {
          if(!this.monitoringResult[socket.CDFILIAL]) {
            this.monitoringResult[socket.CDFILIAL] = {};
          }

          const subsidiaryIndex = this.slavelist.findIndex((item) => {
            return item.codigo === data.CDFILIAL;
          });

          if (this.slavelist[subsidiaryIndex]) {
            this.slavelist[subsidiaryIndex].situacao = 'Ativo';
            this.slavelist[subsidiaryIndex].socketId = socket.id;      
          
            this.monitoringClients.push({
              id: socket.id,
              socket: socket,
              time: new Date().getTime(),
            });

            this.io.to(socket.id).emit('message', result);
            this.io.to(socket.id).emit('command', { type: 'downloadFromUrl', data: { dataUrl: '/API/getStructure' } });

            if (this.portalSocketId) {
              this.io.to(this.portalSocketId).emit('login', {
                text: 'UNIDADE CONECTADA',
                error: false,
                message: {
                  data: this.slavelist,
                },
                result,
              });
            }
          }
        }
      }).catch((error) => {
        console.log(error);
        this.io.to(socket.id).emit('message', 'Você não é um usuário autorizado!');
        socket.disconnect();
      });
    }
  }

  async getDashboardFiliais(data = null) {
    const filiais = [];
    const queryResultAll = await adqGetDashboardFiliais.get(paginacao(data));
    const { rows } = queryResultAll;
    const assConfig = await this.getAssConfig();

    rows.forEach((element) => {
      const filialRow = {
        id: element[0],
        codigo: element[0],
        name: element[1],
        situacao: 'Inativa',
        email: '',
        socketId: '',
        sat: element[3] == 0 ? 'N' : 'E', // Tratamento especial para o Sat: S => Sucesso | E => Error | N => Filial não utiliza SAT
        websocket: '',
        nrorg: element[2],
      };

      //Cria todos os monitoramentos dinâmicos na row para funcionamento do filtro | N => Erro (bolinha vermelha)
      assConfig.forEach((config) => {
        filialRow[config.ID] = 'N'
      });
      filiais.push(filialRow);
    });

    if (this.slavelist !== undefined) { filiais.concat(this.slavelist); }

    return filiais;
  }

  async getAssConfig() {
    const config = [];
    const assConfig = await parametrizacoesModel.getAssConfig(true);
    assConfig.rows.forEach((element) => {
      config.push({
        ID:        element[0].toString(),
        NOME:      element[1],
        DESCRICAO: element[2],
        TIPO:      element[3],
        ORDEM:     element[4],
        ATIVO:     element[5],
        COMANDO:   element[6],
        INTERVALO: element[7]
      });
    });
    return config;
  }
}

module.exports = new SocketService();
