const updaterModel = require('../model/updaterModel');
const updaterProdFiliModel = require('../model/updaterProdFiliModel');
const GeraNovoCodigo = require('../modules/NovoCodigoModule');
const OracleService = require('./OracleService');
const { v4: uuidv4 } = require('uuid');

class UpdaterService {
    async insertEventTable(cdFilial, devopsId, installVersion, createNewVersion, action) {
        let response = {
            error: true,
            message: 'O evento já foi registrado no sistema.'
        };

        let params = {
            P_CDFILIAL: cdFilial,
            P_CDPRODUCTID: devopsId
        }
        
        const unitEvents = await updaterModel.getEventsByUnit(params);

        if (unitEvents.rows.length === 0) {
            const eventSeq = await GeraNovoCodigo.prxUpdaterEvento(cdFilial);
            const uuid = uuidv4();
            
            params = {
                P_CDFILIAL: cdFilial,
                P_CDEVENTO: eventSeq,
                P_CDUUID: uuid,
                P_CDPRODUCTID: devopsId,
                P_IDSTEVENTO: action,
                P_NRVERSAO: installVersion,
                P_IDCRIARVERSAO: createNewVersion
            }
            
            await updaterModel.insertPendingEvent(params);

            const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);
            
            params = {
                P_CDFILIAL: cdFilial,
                P_CDHISTORICO: historySeq,
                P_CDEVENTO: eventSeq,
                P_CDUUID: uuid,
                P_CDOPERADOR: null,
                P_CDPRODUCTID: devopsId,
                P_NMPRODUCT: null,
                P_IDSTEVENTO: action,
                P_NRVERSAOATUAL: null,
                P_NRVERSAOALVO: installVersion,
                P_IDCRIARVERSAO: createNewVersion
            }
            
            await updaterModel.appendUpdaterHistory(params);

            response = {
                error: false,
                message: 'Evento cadastrado com sucesso.'
            }
            
            await OracleService.commit();
        }

        return response;
    }

    async updateEventTable(cdFilial, devopsId, installVersion, createNewVersion, action) {
        let response = {
            error: true,
            message: 'O evento já foi atualizado no sistema.'
        };

        const eventId = action === 'INSTALACAO_EM_CURSO' ? 'INSTALACAO_PENDENTE' : 'REMOCAO_PENDENTE';

        let params = {
            P_CDFILIAL: cdFilial,
            P_CDPRODUCTID: devopsId,
            P_IDSTEVENTO: eventId
        }
        
        const pendingEvents = await updaterModel.getProductUpdaterEventByEventId(params);

        if (pendingEvents.rows.length > 0) {
            const uuid = uuidv4();

            params = { 
                P_IDSTEVENTO: action,
                P_CDEVENTO: pendingEvents.rows[0].CDEVENTO
            };

            await updaterModel.updateEventInProgress(params);

            const history = await updaterModel.getUpdaterHistoryByEventId(params);

            if (history.rows.length === 0) {
                const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);
                
                params = {
                    P_CDFILIAL: cdFilial,
                    P_CDHISTORICO: historySeq,
                    P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
                    P_CDUUID: uuid,
                    P_CDOPERADOR: null,
                    P_CDPRODUCTID: devopsId,
                    P_NMPRODUCT: null,
                    P_IDSTEVENTO: 'INSTALACAO_EM_CURSO',
                    P_NRVERSAOATUAL: null,
                    P_NRVERSAOALVO: installVersion,
                    P_IDCRIARVERSAO: createNewVersion
                }
    
                await updaterModel.appendUpdaterHistory(params);

                response = {
                    error: false,
                    message: 'Evento atualizado com sucesso. Histórico criado'
                }
            }
            await OracleService.commit();
        }

        return response;
    }

    async finishEventTable(cdFilial, devopsId, installVersion, createNewVersion) {
        let response = {
            error: true,
            message: 'O evento já foi atualizado no sistema.'
        };

        let params = {
            P_CDFILIAL: cdFilial,
            P_CDPRODUCTID: devopsId
        }
        
        const inProgressEvents = await updaterModel.getInProgressEvents(params);

        if (inProgressEvents.rows.length > 0) {
            const uuid = uuidv4();

            params = { 
                P_IDSTEVENTO: 'FINALIZADO',
                P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO
            };

            await updaterModel.updateEventInProgress(params);

            const history = await updaterModel.getUpdaterHistoryByEventId(params);

            if (history.rows.length === 0) {
                const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);
                
                params = {
                    P_CDFILIAL: cdFilial,
                    P_CDHISTORICO: historySeq,
                    P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
                    P_CDUUID: uuid,
                    P_CDOPERADOR: null,
                    P_CDPRODUCTID: devopsId,
                    P_NMPRODUCT: null,
                    P_IDSTEVENTO: 'FINALIZADO',
                    P_NRVERSAOATUAL: null,
                    P_NRVERSAOALVO: installVersion,
                    P_IDCRIARVERSAO: createNewVersion
                }
    
                await updaterModel.appendUpdaterHistory(params);

                response = {
                    error: false,
                    message: 'Evento atualizado com sucesso. Histórico criado'
                }
            }
            await OracleService.commit();
        }

        return response;
    }

    async updateProdFiliVersion(updateVersion, cdFilial, devopsId) {
        const params = { 
            NRVERSAO: updateVersion,
            CDFILIAL: cdFilial,
            UPDATERPRODID: devopsId
        };

        await updaterProdFiliModel.updateUpdaterProdFiliVersion(params);

        await OracleService.commit();
    }

    async cancelUnitEvents(cdFilial) {
        let params = {
            P_CDFILIAL: cdFilial
        }
        
        await updaterModel.cancelUnitEvents(params);
        
        await OracleService.commit();

        return response;
    }

    async cancelEventTable(cdFilial, devopsId, installVersion, createNewVersion) {
        let response = {
            error: true,
            message: 'O evento já foi atualizado no sistema.'
        };

        let params = {
            P_CDFILIAL: cdFilial,
            P_CDPRODUCTID: devopsId
        }
        
        const unitEvents = await updaterModel.getEventsByUnit(params);

        if (unitEvents.rows.length > 0) {
            const uuid = uuidv4();

            params = { 
                P_IDSTEVENTO: 'CANCELADO',
                P_CDEVENTO: unitEvents.rows[0].CDEVENTO
            };

            await updaterModel.updateEventInProgress(params);

            const history = await updaterModel.getUpdaterHistoryByEventId(params);

            if (history.rows.length === 0) {
                const historySeq = await GeraNovoCodigo.prxUpdaterHistorico(cdFilial);
                
                params = {
                    P_CDFILIAL: cdFilial,
                    P_CDHISTORICO: historySeq,
                    P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
                    P_CDUUID: uuid,
                    P_CDOPERADOR: null,
                    P_CDPRODUCTID: devopsId,
                    P_NMPRODUCT: null,
                    P_IDSTEVENTO: 'CANCELADO',
                    P_NRVERSAOATUAL: null,
                    P_NRVERSAOALVO: installVersion,
                    P_IDCRIARVERSAO: createNewVersion
                }
    
                await updaterModel.appendUpdaterHistory(params);

                response = {
                    error: false,
                    message: 'Evento atualizado com sucesso. Histórico criado'
                }
            }
            await OracleService.commit();
        }

        return response;
    }

    async getPendingEvents() {
        return updaterModel.getPendingEvents();
    }

    async getPendingEventsByUnit(cdFilial) {
        let params = {
            P_CDFILIAL: cdFilial
        }
        
        return updaterModel.getPendingEventsByUnit(params);
    }
}

module.exports = new UpdaterService()