import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { AlertService, DialogService, Form, LoadingService, Modal, ModalService, Select, Text, TextInput } from '@zeedhi/common';
import { Highlight } from '@zeedhi/zd-highlight-common';
import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-sql.min';

export class ParametrizacoesController {
  private paramEditModal!: Modal;

  public isVisibleRowHighlight = true;

  private dataSourceEdit = [{
      value: 'I',
      text: 'Assistant (Proprietário)',
  }, {
      value: 'A',
      text: 'Base de Dados - Cloud',
  }, {
      value: 'L',
      text: 'Base de Dados - Loja',
  }, {
      value: 'C',
      text: 'Apache - Cloud',
  }, {
      value: 'E',
      text: 'Apache - Loja',
  }, {
      value: 'S',
      text: 'Serviços - Loja',
  }];

  private dataSourceInsert = [{
      value: 'A',
      text: 'Base de Dados - Cloud',
  }, {
      value: 'L',
      text: 'Base de Dados - Loja',
  }, {
      value: 'C',
      text: 'Apache - Cloud',
  }, {
      value: 'E',
      text: 'Apache - Loja',
  }, {
      value: 'S',
      text: 'Serviços - Loja',
  }];

  public async gridMounted() {
		this.paramEditModal = await ModalService.createFromJson('parametrizacoes-edit', true);
  }

	public gridBeforeDestroy() {
		this.paramEditModal.destroy();
	}

  public loadLanguage() {
    const { code, language } = Metadata.getInstance<Highlight>('COMANDO');
    return Prism.highlight(code || '', Prism.languages[language], language);
  }

	public async gridCellClick({ component, row } : any) {
		if (!component.editing) {
			await this.paramEditModal.show();
			const form = Metadata.getInstance<Form>('edit-modal-form');
      const typeColumn: Select = form.getChildInstance('TIPO');
      const nameColumn: TextInput = form.getChildInstance('NOME');
      const commandObsColumn: Text = form.getChildInstance('COMMAND_OBS');
      form.value = row;
      if (row.TIPO === 'I') {
        this.isVisibleRowHighlight = false;
        typeColumn.datasource.updateData(this.dataSourceEdit);
        typeColumn.readonly = true;
        nameColumn.readonly = true;
        commandObsColumn.text = 'Comandos internos não podem ser alterados';
      } else {
        typeColumn.datasource.updateData(this.dataSourceInsert);
        typeColumn.readonly = false;
        nameColumn.readonly = false;
        this.changeModalFormFields(row.TIPO);
      }
		}
	}

  public changeModalTipo(params: any) {
    this.changeModalFormFields(params.value);
  }

  private changeModalFormFields(type: any) {
    const form = Metadata.getInstance<Form>('edit-modal-form');
    const commandApacheColumn: TextInput = form.getChildInstance('COMANDO_APACHE');
    const commandServicesColumn: TextInput = form.getChildInstance('COMANDO_SERVICOS');
    const commandObsColumn: Text = form.getChildInstance('COMMAND_OBS');

    commandApacheColumn.removeValidation('required');
    commandServicesColumn.removeValidation('required');

    if (type === 'C' || type === 'E') {
      this.isVisibleRowHighlight = false;
      commandServicesColumn.isVisible = false;
      commandApacheColumn.isVisible = true;
      commandApacheColumn.addValidation('required', {});
      form.value.COMANDO_APACHE = form.value.COMANDO;
      commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando a requisição GET feita para os endereços informados funcionar.';
    } else if (type === 'S') {
      this.isVisibleRowHighlight = false;
      commandApacheColumn.isVisible = false;
      commandServicesColumn.isVisible = true;
      commandServicesColumn.addValidation('required', {});
      form.value.COMANDO_SERVICOS = form.value.COMANDO;
      commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando todos os serviços informados estiverem em execução.';
    } else if (type !== 'I') {
      this.isVisibleRowHighlight = true;
      commandServicesColumn.isVisible = false;
      commandApacheColumn.isVisible = false;
      const commandComponent = Metadata.getInstance<Highlight>('COMANDO');
      commandComponent.code = form.value.COMANDO;
      commandObsColumn.text = 'Esse monitoramento será considerado como sucesso quando a consulta não retornar nenhum registro.';
    }
  }

	public closeParamEditModal() {
		this.paramEditModal.hide();
	}

  public async addClick() {
    await this.paramEditModal.show();
    const form = Metadata.getInstance<Form>('edit-modal-form');
    const typeColumn: Select = form.getChildInstance('TIPO');
    typeColumn.datasource.updateData(this.dataSourceInsert);
    typeColumn.readonly = false;
    form.value.TIPO = 'A';
  }

  public getConditionalCssClass(param: any) {
    return param.row.ATIVO === 'N' ? 'ass-inactive' : '';
  }

  public async updateInModal() {
    const grid = Metadata.getInstance<TekGrid>('gridParams');
    const form = Metadata.getInstance<Form>('edit-modal-form');
    const commandComponent = Metadata.getInstance<Highlight>('COMANDO');
    if (!form.validate()) return;
    if (!commandComponent.code && (form.value.TIPO === 'A' || form.value.TIPO === 'L')) {
      this.showAlert('Informe o comando', 'warning');
      return;
    }

    if (form.value.TIPO === 'I') {
      form.value.COMANDO = 'INTERNO';
    } else if (form.value.TIPO === 'A' || form.value.TIPO === 'L') {
      form.value.COMANDO = commandComponent.code;
    } else if (form.value.TIPO === 'C' || form.value.TIPO === 'E') {
      form.value.COMANDO = form.value.COMANDO_APACHE;
    } else {
      form.value.COMANDO = form.value.COMANDO_SERVICOS;
    }
    const rq = form.value.ID ? `connect/portal/${form.value.ID}` : 'insertAssConfig';
    const conexao = process.env.VUE_APP_METADATA_END_POINT + rq;

    LoadingService.show();
    Http.put(conexao, form.value).then(() => {
      LoadingService.hide();
      this.paramEditModal.hide();
      grid.datasource.get();
      const message = form.value.ID ? 'Registro atualizado com sucesso!' : 'Registro inserido com sucesso!';
      this.showAlert(message, 'success');
    }).catch((error) => {
      LoadingService.hide();
      this.showAlert(error, 'error');
    });
	}

  public async beforeDelete(params: any) {
    params.event.preventDefault();
    const grid = Metadata.getInstance<TekGrid>('gridParams');

    DialogService.show({
			name: `confirm_${grid.name}`,
			type: 'warning',
			title: 'Exclusão',
			text: 'Deseja realmente excluir os registros selecionados?',
			buttons: [
				{
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => DialogService.hide(),
          },
        },
				{
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.deleteConfig,
        },
			],
		});
  }

  private deleteConfig: any = {
    click: () => this.doBeforeDelete(),
  };

  private async doBeforeDelete() {
    const grid = Metadata.getInstance<TekGrid>('gridParams');

    const ids: any = [];
    grid.selectedRows.forEach((element) => {
      if (element.ID) {
        ids.push(element.ID);
      }
    });

    const rq = `connect/portal/${ids.join(',')}`;
    const conexao = process.env.VUE_APP_METADATA_END_POINT + rq;

    LoadingService.show();
    Http.delete(conexao).then(() => {
      this.paramEditModal.hide();
      grid.datasource.get();
      const message = 'Registros removidos com sucesso!';
      grid.selectedRows = [];
      this.showAlert(message, 'success');
    }).catch((error) => {
      this.showAlert(error, 'error');
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
    });
  }

  private showAlert(message: string, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 5000,
    });
  }
}
