import { Modal, ModalService } from '@zeedhi/common';

import { SocketEvent } from '@/socket/socket-event';
import {
  Config,
  IDictionary,
  IEventParam,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';

export class SalesController {
  public socket?: Socket;

  public filialId: any;

  public title: any;

  private fullscreenModal!: Modal;

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('sales', true);
  }

  public destroyFullscreenModal() {
    this.fullscreenModal?.destroy();
  }

  public openFullscreenModal(row:any) {
    // console.warn(row);
    this.filialId = row.row;
    this.title = 'Monitoramento de Vendas da ';
    this.title += row.row.name;
    this.title += ' - ';
    this.title += row.row.id;
    if (row.row.sat !== true) {
      this.fullscreenModal.show();
    }
    this.fullscreenModal.show();
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }

  /** Status do Vendas */
  public getConditionalStyleMesa({ row }: IDictionary<any>) {
    if (row.mesa === true) {
      return 'green';
    }
    return 'red';
  }

  /** Status do Vendas */
  public getConditionalStyleComanda({ row }: IDictionary<any>) {
    if (row.comanda === true) {
      return 'green';
    }
    return 'red';
  }

  /** Status do Vendas */
  public getConditionalStyleDelivery({ row }: IDictionary<any>) {
    if (row.delivery === true) {
      return 'green';
    }
    return 'red';
  }

  /** Status do Vendas */
  public getConditionalMessage({ row }: IDictionary<any>) {
    let message = '';
    if (row.vendas === true) {
      message = 'Não foram localizados erros com este pedido.';
    } else if (row.message === '') {
      message = 'Este pedido não foi sincronizado com o Oracle.';
    } else {
      message = row.message;
    }
    return message;
  }

  /**
   * Método executado quando o grid é criado
   */
  public gridMounted({ component }: IEventParam<TekGrid>) {
    // inicializa o socket
    this.socket = io(Config.endPoint);

    this.socket.off(SocketEvent.CONNECTION)
      .emit('sales', {
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('sales', async (branc: IDictionary) => {
        await component.datasource.updateData(branc.message.vendas);
        await component.datasource.get();
      });
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {

  }
}
