/* eslint-disable */
import { AlertService, LoadingService, Modal, ModalService, Grid, TreeGrid, Button, Select, IButtonEvent, DialogService } from "@zeedhi/common";
import { Config, Http, IEventParam, Metadata } from "@zeedhi/core";
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { io, Socket } from "socket.io-client";
import { SocketEvent } from "@/socket/socket-event";

export class UpdatesController {
  public socket?: Socket;

  private fullscreenModal!: Modal;

  private defaultGridTitle!: String;

  private filterProductName!: String;

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson("historico", true);
  }

  public openFullscreenModal({ row }: any) {
    LoadingService.show();
    const filter = { 
      params: { 
        CDFILIAL: row.CDFILIAL,
        CDPRODUCTID: row.ID,
      } 
    };
    Http.get('/updaterProdutoHistorico', filter).then((result) => {
      const grid = this.fullscreenModal.children[1];
      grid.datasource.data = result.data.data;
      this.fullscreenModal.show();
    }).catch((error) => {
      this.showAlert(error, "error");
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }
  
  public async gridMounted(force: boolean = false) {   
    if (!this.socket) {
      this.socket = io(Config.endPoint);
    }

    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);
    this.socketListener();
     
    const grid = Metadata.getInstance<TekGrid>("updater-grid");
    const date = new Date().toLocaleDateString();
    const savedDate = localStorage.getItem("LAST_DATE_UPDATER");
    this.defaultGridTitle = grid.title;

    if (savedDate !== date || (typeof force === "boolean" && force)) {
      LoadingService.show("Atualizando lista de sistemas...");
      const conexao = `${Config.metadataEndPoint}orgProducts`;      
      Http.get(conexao).then((result) => {
        if (result.data.isError) {
          this.showAlert(`Erro ao atualizar a lista de sistemas: ${result.data.response}`, "error");
        } else {
          this.showAlert("Lista de sistemas atualizada com sucesso!", "success");
        }
      }).catch((error) => {
        this.showAlert(error, "error");
      }).finally(() => {
        LoadingService.hide();
        localStorage.setItem("LAST_DATE_UPDATER", date);
        if(typeof force !== "boolean") {
          this.openFilter(grid);
        }
      });
    } else {
      this.openFilter(grid);  
    }
  }

  private openFilter(grid: TekGrid) {
    const filterButton = Metadata.getInstance<Button>(`${grid.name}_filterButton`); 
    filterButton.click({} as Event, {} as HTMLElement);
  }

  public updateSystemList() {
    this.gridMounted(true);
  }

  private showAlert(message: string, colorStr: string) {
    AlertService.show({
      name: "info-alert",
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }

  public executeDelete(row: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
      LoadingService.show();
      const params = {
        CDFILIAL: row.CDFILIAL,
        DEVOPSID: row.ID.toString(),
        INSTALLVERSION: row.NRVERSAOATUAL,
        CREATENEWVERSION: row.IDUTMULTVER,
        TYPE: 'REMOVE'
      }
      Http.post('/updater/removeVersion', params).then((result) => {
        if (result.data && result.data.error) {
          this.showAlert(result.data.message, 'error');
        } else {
          this.socket?.emit('execUpdater', params);
          this.showAlert('Requisição de remoção enviada com sucesso!', 'success');
          grid.datasource.get();
        }
      }).catch((error) => {
        this.showAlert(error, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
  }

  public deleteVersion({ row }: any) {
    if (!row.CONNECTION_STATUS) {
      this.showAlert('A conexão deve estar ativa para realizar o processo.', '#e68129');
    } else if (row.IDSTEVENTO === 'INSTALACAO_PENDENTE') {
      this.showAlert('Sistema não instalado.', '#e68129');
    } else {
      DialogService.show({
        name: 'remove-dialog',
        type: 'warning',
        title: 'Exclusão',
        text: 'Deseja realmente excluir essa versão desse sistema?',
        buttons: [
          { 
            name: 'no', 
            component: 'ZdButton', 
            label: 'Não',
            outline: true,
            events: { 
              click: () => { 
                DialogService.hide(); 
              } 
            },
          },
          { 
            name: 'yes', 
            component: 'ZdButton', 
            label: 'Sim', 
            events: { 
              click: () => { 
                DialogService.hide(); 
                this.executeDelete(row); 
              } 
            } 
          },
        ],
      });
    }
  }

  private async executeInstall(row: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    LoadingService.show();
    const params = {
      CDFILIAL: row.CDFILIAL,
      DEVOPSID: row.ID.toString(),
      INSTALLVERSION: row.VERSION.NRVERSAO,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'INSTALL'
    }
    Http.post('/updater/installVersion', params).then((result) => {
      if (result.data && result.data.error) {
        this.showAlert(result.data.message, 'error');
      } else {
        this.socket?.emit('execUpdater', params);
        this.showAlert('Requisição de instalação/atualização enviada com sucesso!', 'success');
        grid.updateGroupedData(false);
      }
    }).catch((error) => {
      this.showAlert(error, 'error');
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public installVersion({ row }: any) {
    if (!row.VERSION) {
      this.showAlert('Selecione a versão.', '#e68129');
    } else if (!row.CONNECTION_STATUS) {
      this.showAlert('A conexão deve estar ativa para realizar o processo.', '#e68129');
    } else if (row.VERSION.IDDOWNLOADVER === 'N') {
      this.showAlert('Essa versão ainda não está disponível para download. Aguarde alguns instantes e tente novamente.', '#e68129');
    } else if (row.VERSION.IDDOWNLOADVER === 'E') {
      this.showAlert('Ocorreu um erro ao realizar o download dessa versão para a nuvem. Atualize a lista de sistemas, aguarde alguns instantes e tente novamente.', '#e68129');
    } else {
      DialogService.show({
        name: 'install-dialog',
        type: 'warning',
        title: 'Instalação',
        text: 'Deseja realmente instalar/atualizar esse sistema?',
        buttons: [
          { 
            name: 'no', 
            component: 'ZdButton', 
            label: 'Não',
            outline: true,
            events: { 
              click: () => { 
                DialogService.hide(); 
              } 
            },
          },
          { 
            name: 'yes', 
            component: 'ZdButton', 
            label: 'Sim', 
            events: { 
              click: () => { 
                DialogService.hide(); 
                this.executeInstall(row); 
              } 
            } 
          },
        ],
      });
    }
  }

  private async socketListener() {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    
    this.socket?.on('updaterInProgress', (result) => {
      let message = 'Instalação iniciada...'
      if(result == 'UPDATE') {
        message = 'Atualização iniciada...';
      } else if(result == 'REMOVE') {
        message = 'Remoção iniciada...';
      }
      this.showAlert(message, 'success');
      grid.datasource.get();
    });

    this.socket?.on('updaterFinish', (result) => {
      if(result == 'ERROR') {
        this.showAlert('Ocorreu um erro durante a execução do último processo. Verifique o histórico para mais informações.', 'error');
      } else {
        this.showAlert('Operação finalizada com sucesso!', 'success');
      }
      grid.datasource.get();
    });

    this.socket?.on('disconnect', () => {
      this.showAlert('A página será recarregada para restabelecer a conexão...', 'error');
      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });

    this.socket?.on('updaterConnected', async () => {
      grid.datasource.get();
    });
  }
 
  public setAvailableVersionFilter({ component }: any) {
    const grid = Metadata.getInstance<TekGrid>("updater-grid");
    component.datasource.filter = grid.datasource.filter;
  }
  
  public beforeApplyFilter({ component }: any) {
    // Force to reload all available version column
    component.datasource.updateData([]);
    component.toolbarSlot[0].text = `${this.defaultGridTitle} - ${this.filterProductName}`;
  }
  
  public changeProductId({ component, value}: any) {
    const data = component.datasource.data.find((item: any) => item.ID === value);
    if (data) {
      this.filterProductName = data.NMPRODUCT;
    }
  }

  public gridBeforeDestroy() {
    this.socket?.removeAllListeners('updaterInProgress');
    this.socket?.removeAllListeners('updaterFinish');
    this.socket?.removeAllListeners('updaterConnected');
    this.socket?.removeAllListeners('disconnect');
    this.fullscreenModal?.destroy();
  }

  public getConnectionColor({ row }: any) {
    return row.CONNECTION_STATUS ? 'green' : 'red';
  }

  public changeSelectVersion({ value }: IEventParam<Select>) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    grid.datasource.currentRow.VERSION = value;
  }
}
